/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Luigi Zangrando <zangrando@pd.infn.it>
 *
 */

package org.glite.ce.monitor;

import org.apache.log4j.Logger;
import org.glite.ce.faults.BaseFaultType;


public class LogUtils {
   private static Logger logger = Logger.getLogger(CEMonitorService.class.getName());

   /**
    * Logs an error message to the default CREAM logger
    * 
    * @param msg the message text
    */
   public static void logError(String msg) {
	   logger.error(msg);
   }


   /**
    * Logs a debug message to the default CREAM logger
    * 
    * @param msg the message text
    */
   public static void logDebug(String msg) {
	   logger.debug(msg);
   }


   public static void printFault(BaseFaultType fault) {
      String s = "\n-------------------------------------------------------";
      s += "\nclass name: " + fault.getClass().getName();
      s += "\ntimestamp: " + fault.getTimestamp().getTime();
      s += "\nmethod name: " + fault.getMethodName();
      s += "\ndescription: " + fault.getDescription();
      s += "\nfault cause: " + fault.getFaultCause(); 
      s += "\n-------------------------------------------------------";
                  
      logger.error(s);
   }

}
