/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Author Luigi Zangrando <zangrando@pd.infn.it>
 *
 */

package org.glite.ce.monitor.action;

import org.glite.ce.monitorapij.queryprocessor.QueryResult;
import org.glite.ce.monitorapij.resource.types.Action;
import org.glite.ce.monitorapij.resource.types.Parameter;
import org.glite.ce.monitorapij.resource.types.Property;
import org.glite.ce.monitorapij.types.Event;
import org.glite.ce.monitorapij.types.Notification;

import java.util.Calendar;


/**
 * This class is an extension of <code>Action</code> class.<br>
 * It is used to mark an ISM_LDIF formatted message as valid, not expired, message.
 * This is done by adding a TTLCEinfo=300 field to the LDIF message, so as the purchaser 
 * will interpret this message as valid.
 */
public class SendNotificationAction extends Action {
    private static final long serialVersionUID = 1L;    
    
    /**
     * Creates a new SendNotificationAction object.
     */
    public SendNotificationAction() {
        super("SendNotification");

       // Property[] properties = new Property[] { new Property("TTLCEinfo", "300") };
       // setProperty(properties);
/*        Property[] properties = new Property[2];
        properties[0] = new Property("supportedDialectsList", "LFID,ISM_LDIF,ISM_LDIF_GLUE_1.2,OLD_CLASSAD,NEW_CLASSAD,ISM_CLASSAD,ISM_CLASSAD_GLUE_1.2");
        properties[1] = new Property("TTLCEinfo", "300");
        
        setProperty(properties);
*/
    }

    /**
     * Check if this dialect is defined in to the supportedDialectsList parameter.
     *
     * @param dialect The dialect String to be checked.
     *
     * @throws Exception This exception is thrown if the specified dialect is null 
     * or is not ISM_LDIF or ISM_CLASSAD.
     */
/*
    private void checkDialect(String dialect) throws Exception {
        if (dialect == null) {
            throw (new Exception("invalid dialect!"));
        }

        Property property = getProperty("supportedDialectsList");
        
        if(property != null) {
           String[] dialects = property.getValue().split(",");   
                          
           for (int i = 0; i < dialects.length; i++) {
              if(dialect.equalsIgnoreCase(dialects[i])) {
                 return;
              }
           }         
        }
        throw (new Exception("invalid dialect! (\"" + dialect + "\")"));
    }
*/

    /**
     * This method is called by <code>NotificationHolder</code> 
     * specifying a <code>Notification</code> and a <code>QueryResult</code> array.<br>
     * It performs the specific operations of this action:<br>
     * The dialect is read from the topic of the specified notification, and then it is checked 
     * using checkDialect() method to verify if it is a ISM_LDIF dialect.<br>
     * Then, for each successful QueryResult, the relative sensor event is called, and then its 
     * message is changed adding the TTLCEInfo information. 
     *
     * @param The specified notification.
     * @param result The specified <code>QueryResult</code> array.
     *
     * @throws Exception DOCUMENT ME!
     */

    public synchronized void execute() throws Exception {
        Parameter parameter = getParameter("notification");

        if(parameter == null) {
            return;
        }

        Notification notification = (Notification)parameter.getValue();

        parameter = getParameter("queryResult");

        if(parameter == null) {
            return;
        }
        
        QueryResult[] result = (QueryResult[])parameter.getValue();
        Event[] eventArray = notification.getEvent();
        
        if((result == null || result.length == 0 ) && (eventArray == null || eventArray.length == 0 )) {
            parameter = getParameter("subscriptionId");
            if(parameter == null) {
                return;
            }

            if(notification.getTopic().getName().equalsIgnoreCase("CREAM_JOBS")) {
                String classad = "[ SUBSCRIPTION_ID = \"" + parameter.getValue() + "\";\nKEEP_ALIVE = true; ]";
                eventArray = new Event[1]; 
                eventArray[0] = new Event(-1, Calendar.getInstance(), new String[] { classad }, "CREAM Job Sensor");
            
                notification.setEvent(eventArray);            
            }
        }

        
        /*
        Event[] eventArray = notification.getEvent();
        
        for (int i = 0; i < eventArray.length; i++) {
            Event event = eventArray[i];
            QueryResult eventResult = result[i];

            for (int j = 0; j < eventResult.size(); j++) {
                if(eventResult.isSuccessfully(j) == this.isDoActionWhenQueryIs()) {
                    String msg = event.getMessage(j);
                    event.setMessage(j, msg);
                }
            }
        }
        
        notification.setEvent(eventArray);
        */
/*
        boolean isISM_LDIF = dialect.toLowerCase().indexOf("ldif") == -1? false: true;
                
        Event[] eventArray = notification.getEvent();
        
        String ttlCEInfo = getProperty("TTLCEinfo").getValue();
        
        for (int i = 0; i < eventArray.length; i++) {
            Event event = eventArray[i];
            QueryResult eventResult = result[i];

            for (int j = 0; j < eventResult.size(); j++) {
                if(eventResult.isSuccessfully(j) == this.isDoActionWhenQueryIs()) {
                    String msg = event.getMessage(j);

                    int index = msg.indexOf("TTLCEinfo");
                    if(index > -1) {
                        msg = msg.substring(0, index);
                    }
                    
                    if(isISM_LDIF) {
                        if(j > 0) {
                            msg += "\nTTLCEinfo: " + ttlCEInfo;
                        }
                    } else {
                        msg = msg.substring(0, msg.length() - 2);
                        msg += "\n\tTTLCEinfo = " + ttlCEInfo + ";\n]";
                    }

                    event.setMessage(j, msg);
                }
            }
        }
        */
    }
}
