/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.monitor.configuration.basic;

import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.NoInitialContextException;
import javax.naming.spi.InitialContextFactory;

import org.glite.ce.commonj.configuration.basic.ConfigDataBase;

//import org.apache.log4j.Logger;

public class CEMonConfigContextFactory implements InitialContextFactory {
	//private static Logger logger = Logger.getLogger(CEMonConfigContextFactory.class.getName());

	private static Hashtable initCtxTable = new Hashtable();

	public CEMonConfigContextFactory() {
	}

	public Context getInitialContext(Hashtable env) throws NamingException {
		String url = (String) env.get(Context.PROVIDER_URL);
		if (url == null)
			throw new NoInitialContextException();
		  
		String confFilename = null;
		try {
			confFilename = (new File(url)).getCanonicalPath();
		} catch (IOException ex) {
			throw new NoInitialContextException("Bad provider url");
		}

		ConfigDataBase backend = null;
		if (initCtxTable.containsKey(url)) {
			backend = (ConfigDataBase) initCtxTable.get(url);
		} else {
			InputStream in = null;
			try {
				in = getClass().getResourceAsStream("configuration.properties");
				Properties resource = new Properties();
				resource.load(in);

				backend = new ConfigDataBase(url, resource);
				/*
				 * warning url is not a good key with hard links different urls
				 * may be confliting
				 */
				initCtxTable.put(url, backend);
			} catch (IOException ioEx) {
				throw new NoInitialContextException(ioEx.getMessage());
			} finally {
				if (in != null)
					try {
						in.close();
					} catch (Exception ex) {
					}
			}
		}

		return new CEMonConfigContext(backend);
	}

}
