/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.monitor.configuration.basic;

import java.util.Hashtable;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import javax.naming.Context;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.SearchResult;
import javax.naming.directory.InitialDirContext;

import org.glite.ce.monitorapij.types.CEMonResource;
import org.glite.ce.monitorapij.types.Property;
import org.glite.ce.monitorapij.types.Subscription;
import org.glite.ce.monitorapij.types.Policy;
import org.glite.ce.monitorapij.types.Action;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.PropertyConfigurator;

public class ConfigMain{

    private static void printAttributes(DirContext ctx, String name)
        throws NamingException{
	    NamingEnumeration rootAttrs = ctx.getAttributes(name).getAll();
	    while( rootAttrs.hasMore() ){
            Attribute attr = (Attribute)rootAttrs.next();
            Object tmpo = attr.get(0);
            if( tmpo instanceof String || tmpo instanceof Long )
                System.out.println(attr.getID() + "=" + tmpo.toString());
            else
                System.out.println(attr.getID() + "=" + tmpo.getClass().getName());
	    }
    }

    private static void printContent(DirContext ctx, String name)
        throws NamingException{
        Object obj = ctx.lookup(name);
        if( obj instanceof Subscription ){
            Subscription subscr = (Subscription)obj;
            System.out.println("  Subscription: " + subscr.getName()); 
            Property[] props = subscr.getProperty();
            for(int r=0; props!=null && r<props.length; r++){
                System.out.println("                  " + props[r].getName()
                                   + "=" + props[r].getValue());
            }

            Policy policy = subscr.getPolicy();
            Action[] actions = policy.getAction();

            for(int k=0; k<actions.length; k++){
                System.out.println("                  Actions: " + actions[k].getName());
            }

        }else if( obj instanceof CEMonResource ){
            CEMonResource resource = (CEMonResource)obj;
            System.out.println("  CEMonResource: " + resource.getName()
                               + " (" + resource.getClass().getName() + ")");
            Property[] props = resource.getProperty();
            for(int r=0; props!=null && r<props.length; r++){
                System.out.println("                  " + props[r].getName()
                                   + "=" + props[r].getValue());
            }
        }
    }

    public static void main(String[] args){

        if( System.getProperty("log4j.configuration")!=null )
            PropertyConfigurator.configure(System.getProperty("log4j.configuration"));
        else
            BasicConfigurator.configure();

        try{
            Hashtable env = new Hashtable();
            env.put(Context.INITIAL_CONTEXT_FACTORY, CEMonConfigContextFactory.class.getName());
            env.put(Context.PROVIDER_URL, args[0]);

            DirContext rootCtx = (DirContext)new InitialDirContext(env);

            BufferedReader console = new BufferedReader(new InputStreamReader(System.in));

            for(boolean goon=true; goon;){
                System.out.println("[Root]");
                printAttributes(rootCtx, "");
                printContent(rootCtx, "");
	
                NamingEnumeration namesInContext = rootCtx.list("");
                while( namesInContext.hasMore() ){
                    NameClassPair tmpp = (NameClassPair)namesInContext.next();
                    System.out.println("[" + tmpp.getName() + " (" + tmpp.getClassName() + ")]");
                    printAttributes(rootCtx, tmpp.getName());
                    printContent(rootCtx, tmpp.getName());
                }

                Attributes testAttrs = new BasicAttributes();
                testAttrs.put(new BasicAttribute("category","authzchain"));
                NamingEnumeration queryResult = rootCtx.search("",testAttrs);
                System.out.println("[Query category=authzchain]");
                while( queryResult.hasMore() ){
                    SearchResult resultItem = (SearchResult)queryResult.next();
                    System.out.println(resultItem.getName() + " " + resultItem.getClassName());
                }

                System.out.print(">");
                goon = !console.readLine().equals("quit");

            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }


}
