/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Luigi Zangrando <zangrando@pd.infn.it>
 *
 */

package org.glite.ce.monitor.holder;

import org.apache.axis.MessageContext;
import org.apache.axis.encoding.DeserializationContext;
import org.apache.axis.encoding.Deserializer;
import org.apache.axis.encoding.TypeMapping;
import org.apache.axis.message.EnvelopeHandler;
import org.apache.axis.message.SOAPHandler;
import org.apache.axis.utils.XMLUtils;

import org.apache.log4j.Logger;

import org.glite.ce.monitorapij.types.Subscription;

import org.w3c.dom.Element;

import org.xml.sax.InputSource;

import java.io.StringReader;


/**
 * This class is a deserializer and also a serializer for the 
 * XML file containing the predefined subscriptions list.<br>
 * It is an extension of <code>DeserializationContext</code> axis class.
 * It is used by <code>PredefinedSubscriptionHolder</code> to update the subscription list. 
 */
public class SubscriptionHandler extends DeserializationContext {
    private static Logger logger = Logger.getLogger(SubscriptionHandler.class.getName());
    private Deserializer topDeserializer = null;

    /**
     * Creates a new SubscriptionHandler object.<br>
     * The deserializer class for subscriptions is taken from context's type mapping.
     *
     * @param context The axis MessageContext.
     * @param element The <code>Element</code> relative to a subscription coming from the 
     * parsed XML subscription list file.
     *
     * @throws Exception
     */
    public SubscriptionHandler(MessageContext context, Element element) throws Exception {
        super(context, new SOAPHandler());

        TypeMapping tm = context.getTypeMapping();

        topDeserializer = getDeserializerForType(tm.getTypeQName(Subscription.class));

        pushElementHandler(new EnvelopeHandler((SOAPHandler) this.topDeserializer));

        String inputString = XMLUtils.ElementToString(element);

        inputSource = new InputSource(new StringReader(inputString));
    }

    /**
     * Get the object defined as value in Deserializer topDeserializer. 
     * Here this can be considered the  
     *
     * @return The object defined as value in Deserializer topDeserializer or null if topDeserializer is null.
     */
    public Object getValue() {
        return (topDeserializer == null) ? null : topDeserializer.getValue();
    }

    /**
     * Get the parsed <code>Subscription</code> casting the object obtained by the getValue() method.
     *
     * @return The parsed <code>Subscription</code>
     */
    public Subscription getSubscription() {
        Object obj = getValue();

        return (obj == null) ? null : (Subscription) obj;
    }
}


/*
 * import java.net.URLDecoder; import java.text.DateFormat; import
 * java.text.ParseException; import java.text.SimpleDateFormat; import
 * java.util.ArrayList; import java.util.Calendar; import java.util.Date;
 *
 * import org.apache.axis.types.URI; import
 * org.apache.axis.types.URI.MalformedURIException; import
 * org.glite.ce.monitor.SubscriptionPersistent; import
 * org.glite.ce.monitorapij.types._Action; import
 * org.glite.ce.monitorapij.types._Dialect; import
 * org.glite.ce.monitorapij.types._Policy; import
 * org.glite.ce.monitorapij.types._Query; import
 * org.glite.ce.monitorapij.types._Topic; import org.xml.sax.Attributes; import
 * org.xml.sax.helpers.DefaultHandler;
 *
 *
 *
 *
 * public class SubscriptionHandler extends DefaultHandler { private ArrayList
 * actionList; private ArrayList subscriptionList; private _Action action =
 * null; private SubscriptionPersistent subscription = null; private String
 * value = ""; private boolean isInsideDialect = false; private boolean
 * isInsideTopic = false; private boolean isInsideAction = false; private
 * boolean isInsideParameter = false;
 *
 *
 * protected static final String DEFAULT_PARSER_NAME =
 * "org.apache.xerces.parsers.SAXParser";
 *
 *
 * public SubscriptionHandler( ) { super( );
 *
 * actionList = new ArrayList( ); subscriptionList = new ArrayList( ); }
 *
 *
 * public SubscriptionPersistent[] getSubscriptions( ) {
 * SubscriptionPersistent[] array = new
 * SubscriptionPersistent[subscriptionList.size( )]; array =
 * (SubscriptionPersistent[]) subscriptionList.toArray( array );
 *
 * return array; }
 *
 *
 * public void startDocument( ) { //logger.debug( "Start startDocument:
 * "); }
 *
 *
 * public void endDocument( ) { //logger.debug( "Start endDocument: "); }
 *
 *
 * public void startElement( String uri, String localName, String qName,
 * Attributes atts ) { // logger.debug( "Start element: {" + uri + "}" +
 * localName ); if( localName.equalsIgnoreCase( "subscription" ) ) {
 * subscription = new SubscriptionPersistent( ); subscriptionList.add(
 * subscription );
 *
 * if( atts != null ) { for( int i=0; i<atts.getLength(); i++ ) { if(
 * atts.getLocalName( i ).equalsIgnoreCase( "retryCount" ) ) { String retryCount =
 * atts.getValue( i );
 *
 * try { subscription.setRetryCount( Integer.parseInt( retryCount) ); } catch(
 * Exception e ) { // subscription.setRetryCount( 20 ); } } } } } else if(
 * localName.equalsIgnoreCase( "Action" ) ) { isInsideAction = true; action =
 * new _Action( ); actionList.add( action ); } else { isInsideTopic =
 * localName.equalsIgnoreCase( "Topic" ); //isInsideAction =
 * localName.equalsIgnoreCase( "Action" ); isInsideDialect =
 * localName.equalsIgnoreCase( "Dialect" ); isInsideParameter =
 * localName.equalsIgnoreCase( "Parameter" ); }
 *
 * value = ""; }
 *
 *
 * public void endElement( String uri, String localName, String qName ) {
 * //logger.debug( "End element: {" + uri + "}" + localName + qName ); if(
 * value == null ) { logger.debug( "value is null" ); }
 *
 * if( localName == null ) { return; } else if( localName.equalsIgnoreCase(
 * "MonitorConsumerURL" ) ) { try { subscription.setMonitorConsumerURL( new URI(
 * value ) ); } catch( MalformedURIException e ) { e.printStackTrace( ); } }
 * else if( localName.equalsIgnoreCase( "ExpirationTime" ) ) { try { DateFormat
 * dateFormat = new SimpleDateFormat( "yyyy-MM-dd hh:ss:mm.SSS" ); Date date =
 * dateFormat.parse( value ); Calendar calendar = Calendar.getInstance( );
 * calendar.setTime( date ); subscription.setExpirationTime( calendar ); }
 * catch( ParseException e ) { } } else if( localName.equalsIgnoreCase( "rate" ) ) {
 * _Policy policy = getPolicy( ); policy.setRate( Integer.parseInt( value ) ); }
 * else if( localName.equalsIgnoreCase( "expression" ) ) { _Query query =
 * getQuery( ); query.setExpression( URLDecoder.decode( value ) ); } else if(
 * localName.equalsIgnoreCase( "queryLanguage" ) ) { _Query query = getQuery( );
 * query.setQueryLanguage( value ); } else if( localName.equalsIgnoreCase(
 * "Name" ) ) { _Topic topic = getTopic( );
 *
 * if( isInsideDialect ) { _Dialect dialect = new _Dialect( value );
 *
 * if( !topic.checkSupportedDialect( dialect ) ) { _Dialect[] dialects =
 * topic.getDialect( ); _Dialect[] newDialects = null;
 *
 * if( dialects != null ) { newDialects = new _Dialect[dialects.length + 1];
 * newDialects[newDialects.length-1] = dialect;
 *
 * System.arraycopy( dialects, 0, newDialects, 0, dialects.length ); } else {
 * newDialects = new _Dialect[] { dialect }; // dialect.getDeserializer( ) }
 *
 * topic.setDialect( newDialects ); } } else if( isInsideTopic ) {
 * topic.setName( value ); isInsideTopic = false; } } else { isInsideTopic =
 * !localName.equalsIgnoreCase( "Topic" ); isInsideAction =
 * !localName.equalsIgnoreCase( "Action" ); isInsideDialect =
 * !localName.equalsIgnoreCase( "Dialect" ); isInsideParameter =
 * !localName.equalsIgnoreCase( "Parameter" ); }
 *
 * //value = ""; }
 *
 *
 * private _Policy getPolicy( ) { _Policy policy = subscription.getPolicy( );
 * if( policy == null ) { policy = new _Policy( ); subscription.setPolicy(
 * policy ); }
 *
 * return policy; }
 *
 * private _Query getQuery( ) { _Policy policy = getPolicy( ); _Query query =
 * policy.getQuery( ); if( query == null ) { query = new _Query( );
 * policy.setQuery( query ); }
 *
 * return query; }
 *
 *
 * private _Topic getTopic( ) { _Topic topic = subscription.getTopic( ); if(
 * topic == null ) { topic = new _Topic( ); subscription.setTopic( topic ); }
 *
 * return topic; }
 *
 *
 * public void characters( char ch[], int start, int length ) { value += new
 * String( ch, start, length ); }
 *
 *
 *  }
 */
