/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.monitor.listeners;

import java.io.FileInputStream;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletContextAttributeListener;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextAttributeEvent;
import org.w3c.dom.Document;

import org.apache.axis.server.AxisServer;
import org.apache.axis.transport.http.HTTPConstants;
import org.apache.axis.Constants;
import org.apache.axis.EngineConfiguration;
import org.apache.axis.MessageContext;
import org.apache.axis.WSDDEngineConfiguration;
import org.apache.axis.configuration.FileProvider;
import org.apache.axis.deployment.wsdd.WSDDDeployment;
import org.apache.axis.deployment.wsdd.WSDDDocument;
import org.apache.axis.handlers.soap.SOAPService;
import org.apache.axis.utils.XMLUtils;

import org.apache.log4j.Logger;
import org.glite.ce.monitor.CEMonitorService;




public final class MonitorContextListener implements ServletContextListener, ServletContextAttributeListener {
   private static Logger logger = Logger.getLogger(MonitorContextListener.class.getName());
   private static String AXIS_ENGINE = "AxisEngine";
   private InitMonitorService initService;

   public MonitorContextListener() {
   }

   private AxisServer retrieveAxisServer(ServletContext ctx) {
      Object tmpo = ctx.getAttribute(AXIS_ENGINE);

      if( tmpo != null && tmpo instanceof AxisServer ) {
         return (AxisServer)tmpo;
      }

      return null;
   }

   private void storeAxisServer(ServletContext ctx, AxisServer srv) {
      if (srv == null) {
         ctx.removeAttribute(AXIS_ENGINE);
      } else {
         ctx.setAttribute(AXIS_ENGINE, srv);
      }
   }

  
   
   public void contextInitialized(ServletContextEvent event) {
      ServletContext context = event.getServletContext();    
      AxisServer engine = null;

      synchronized (context) {
         engine = retrieveAxisServer(context);
      
         if(engine == null) {
            String webInfPath = context.getRealPath("/WEB-INF");
            
            if(webInfPath != null) {
               try {
                  EngineConfiguration config = new FileProvider(webInfPath,"server-config.wsdd");

                  engine = new AxisServer(config);
                  engine.init();

                  Document doc = XMLUtils.newDocument( new FileInputStream( webInfPath + "/server-config.wsdd" ) );
                
                  WSDDDocument wsddDoc = new WSDDDocument( doc.getDocumentElement() );
                  config = engine.getConfig();
                  
                  if(config instanceof WSDDEngineConfiguration) {
                     WSDDDeployment deployment = ((WSDDEngineConfiguration)config).getDeployment();
                     wsddDoc.deploy(deployment);

                     MessageContext messageContext = new MessageContext(engine);
                      
                     SOAPService soapService = engine.getService("CEMonitor");
                     messageContext.setService( soapService );                     
                     messageContext.setProperty( Constants.MC_HOME_DIR, context.getRealPath("/") );
                     messageContext.setProperty( HTTPConstants.MC_HTTP_SERVLETLOCATION, context.getRealPath("/WEB-INF") );
                     messageContext.setProperty( Constants.MC_CONFIGPATH, context.getRealPath("/WEB-INF") );
                     
                     initService = new InitMonitorService( messageContext );
                  }
                  
                  engine.refreshGlobalOptions();
                  engine.saveConfiguration();

                  storeAxisServer(context,engine);
               } catch(Exception e) {
                  logger.error(e.getMessage(),e);
               }
            }
         }
      }
   }



   public void contextDestroyed(ServletContextEvent event) {
      ServletContext context = event.getServletContext();

      AxisServer engine = retrieveAxisServer(context);

      if(engine != null) {
         synchronized(engine) {
            if (engine != null) {
               logger.debug("Cleaning up the Axis Server");
               engine.cleanup();
               engine = null;
               storeAxisServer(context,null);
            }
         }
      }

      if( initService != null ) {
         initService.cleanup();
      }
   }

   public void attributeAdded(ServletContextAttributeEvent event) {}

   public void attributeRemoved(ServletContextAttributeEvent event) {}

   public void attributeReplaced(ServletContextAttributeEvent event) {}
   


   public class InitMonitorService extends CEMonitorService {
      public InitMonitorService( MessageContext messageContext ) {     
         this.messageContext = messageContext;
         try {
            super.init();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }        
      }

      public void cleanup() {
         logger.debug("Stopping timer");
         if(timer != null) {
            timer.cancel();
         }
      }
   }

}
