/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.monitor.listeners;

import java.util.Hashtable;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletContextEvent;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.glite.ce.monitor.CEMonitorService;

import org.apache.log4j.Logger;
import org.apache.log4j.LogManager;
import org.apache.log4j.PropertyConfigurator;



public class StartUpManager extends CEMonitorService implements ServletContextListener {
    private static Logger logger = Logger.getLogger(StartUpManager.class.getName());

    private static boolean initialized = false;

    public StartUpManager() {
        super();
    }

    public void contextInitialized(ServletContextEvent event) {
        try{
            Hashtable<String, String> env = new Hashtable<String, String>(2);
            env.put(Context.INITIAL_CONTEXT_FACTORY,
                        "org.apache.naming.java.javaURLContextFactory");
            env.put(Context.URL_PKG_PREFIXES, "org.apache.naming");
            Context initCtx = new InitialContext(env);
            Context context = (Context)(initCtx).lookup("java:comp/env");
            String confFilename = (String)context.lookup("log_configuration_file");

            LogManager.resetConfiguration();
            PropertyConfigurator.configure(confFilename);

        }catch(Throwable th){
            th.printStackTrace();
        }

        if (!initialized) {
            logger.debug("contextInitialized() - Initializing CEMon Service...");
            try {
                init();
		        initialized = true;
		        logger.debug("contextInitialized() - Initialization done");

            } catch (Exception e) {
                //e.printStackTrace();
                logger.error("contextInitialized() - super.init() raised an exeption - initialization error: " 
			     + e.getMessage()+". Follow the stack trace: ",e);
		
		        destroy();
		        //logger.debug("contextInitialized() - Initialization failed");
            }
        }
    }

    public void contextDestroyed(ServletContextEvent event) {
         destroy();
    }

}
