/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import java.net.URISyntaxException;
import java.rmi.RemoteException;
import java.util.Calendar;


import org.apache.axis.MessageContext;
import org.apache.axis.server.AxisServer;
import org.apache.axis.types.URI;
import org.apache.axis.types.URI.MalformedURIException;
import org.glite.ce.faults.GenericFault;
import org.glite.ce.monitor.CEMonitorService;
import org.glite.ce.monitor.SubscriptionPersistent;
import org.glite.ce.monitor.holder.ActionHolder;
import org.glite.ce.monitor.holder.PredefinedSubscriptionHolder;
import org.glite.ce.monitor.holder.QueryProcessorHolder;
import org.glite.ce.monitor.holder.SensorHolder;
import org.glite.ce.monitor.holder.TopicHolder;
import org.glite.ce.monitor.queryprocessor.ClassAdProcessor;
import org.glite.ce.monitorapij.resource.Resource;
import org.glite.ce.monitorapij.resource.ResourceEvent;
import org.glite.ce.monitorapij.sensor.SensorEvent;
import org.glite.ce.monitorapij.types.Action;
import org.glite.ce.monitorapij.types.Dialect;
import org.glite.ce.monitorapij.faults.DialectNotSupportedFault;
import org.glite.ce.monitorapij.types.Event;
import org.glite.ce.monitorapij.types.Info;
import org.glite.ce.monitorapij.types.Notification;
import org.glite.ce.monitorapij.types.Policy;
import org.glite.ce.monitorapij.types.Subscription;
import org.glite.ce.monitorapij.faults.SubscriptionFault;
import org.glite.ce.monitorapij.faults.TopicNotSupportedFault;
import org.glite.ce.monitorapij.types.SubscriptionRef;
import org.glite.ce.monitorapij.types.Topic;
import org.glite.ce.monitorapij.types.TopicArray;

import junit.framework.TestCase;


public class CEMonitorServiceTest extends TestCase {


	private static AxisServer axisServer = null;
	private static MessageContext messageContext = null;
	private static CEMonitorService monitorService = null;
	private static Info info = null;
	private static Event event;
	private static Subscription subscription;
	private static URI consumerURI;
	private static TopicHolder topicHolder;
	private static SubscriptionRef subscriptionRef;
	private org.glite.ce.faults.GenericFault genericFault = null;
	private TopicNotSupportedFault topicNotSupportedFault = null;
//	private static final String LOCAL_SENSOR_URI = "file://"+ System.getProperty( "local_sensor_uri" );
	
	public static void main( String[] args ) {
		junit.textui.TestRunner.run( CEMonitorServiceTest.class );
	}


	protected void setUp( ) throws Exception {
		TestUtilities testUtilities = new TestUtilities();
	   	messageContext= testUtilities.getMessageContext();
		topicHolder = new TopicHolder( );
		CEMonitorService.messageContext = messageContext;
		monitorService = new CEMonitorService( );
                monitorService.init();
	}



	public void testCEMonitorService() {
		assertNotNull(monitorService);
	}

//
//	public void testDoOnSensorError() {
//		//TODO Implement doOnSensorError().
//	}


	public void testDoOnSensorEvent( ) {
//		 wait CEMonitor timers to start
		// the current event is loaded by CEMonitorService initializing TestSensor
		try {
			Thread.sleep(10000);
			
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		
		TopicArray topics = null;
		try {
			topics = monitorService.getTopics( );
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}
		Topic topic = topics.getTopic( 0 );
		//System.out.println("Trovati "+topics.size()+" sensori");
		
		try {
			event = (SensorEvent) monitorService.getEvent( topic );
		} catch (TopicNotSupportedFault e2) {
			e2.printStackTrace();
		} catch (DialectNotSupportedFault e2) {
			e2.printStackTrace();
		} catch (GenericFault e2) {
			e2.printStackTrace();
		} catch (RemoteException e2) {
			e2.printStackTrace();
		}
//		System.out.println(event.getMessage(0));
		assertEquals( "TEST", event.getMessage( 0 ) );
		
		
		// now i try to set another event as current event distinguished from the previous one by the different 
		// message carried: Test Event in place of TEST
		
		SensorEvent nullEvent = null;
		// case event null
		monitorService.doOnSensorEvent( nullEvent );

		// case event source = null
		TestSensorEvent testSensorEvent = new TestSensorEvent( null, System.currentTimeMillis( ), new String[]{"Test Event"} );
		monitorService.doOnSensorEvent( testSensorEvent );

		//    create a sensor and an event
		TestSensor testSensor = new TestSensor( );
		testSensor.setName( "Test Sensor" );
		testSensor.setType("Test Sensor Type");
		// il sensor Type diventa il topic name
		testSensorEvent = new TestSensorEvent( testSensor, System.currentTimeMillis( ), new String[]{"Test Event"} );
		
		
		

//		System.out.println(testSensorEvent.getSource().toString());
		monitorService.doOnSensorEvent( testSensorEvent );
	//	current event should now be setted to the new testSensorEvent
		
		try {
			event = (SensorEvent) monitorService.getEvent( topic );
		} catch (TopicNotSupportedFault e2) {
			e2.printStackTrace();
		} catch (DialectNotSupportedFault e2) {
			e2.printStackTrace();
		} catch (GenericFault e2) {
			e2.printStackTrace();
		} catch (RemoteException e2) {
			e2.printStackTrace();
		}
//		System.out.println(event.getMessage(0));
		assertEquals( "TEST", event.getMessage( 0 ) );
		
		

		testSensor.destroySensor();
		monitorService = null;
	}
//


//
	public void testPing() {
		boolean noExceptionsFound = true;
		try {
			monitorService.ping( );
		} catch( RemoteException e ) {
			noExceptionsFound = false;
			e.printStackTrace( );
		}
		assertTrue( noExceptionsFound );
	}

	public void testGetInfo() {
		try {
			info = monitorService.getInfo( );
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}

		assertNotNull( info );
		assertEquals( "1.0", info.getVersion( ) );
		assertEquals( "First CE-Monitor prototype!", info.getDescription( ) );
		
		// setting monitorVersion property to null
		messageContext.setProperty( "monitorVersion","" );
		CEMonitorService.messageContext = messageContext;
		// create an instance of CEMonitorService
		monitorService = new CEMonitorService( );

		try {
			info = monitorService.getInfo( );
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}
		assertNotNull( info );
		assertEquals( "unknown", info.getVersion( ) );
	}

	/*
	 * Class under test for SubscriptionRef subscribe(Subscription)
	 */
	public void testSubscribe_Subscription() {
		SubscriptionFault subscriptionFault = null;
		// case subscription null
		subscription = null;
		try {
			subscriptionRef = monitorService.subscribe( subscription );
		} catch( SubscriptionFault e ) {
			subscriptionFault = e;
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}
		assertEquals( "the subscription is null", subscriptionFault.getDescription() );

		// case Monitor Consumer URL not specified
		subscription = new Subscription( );
		subscriptionFault = null;
		try {
			subscriptionRef = monitorService.subscribe( subscription );
		} catch( SubscriptionFault e ) {
			subscriptionFault = e;
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}
		assertEquals( "MonitorConsumerURL not specified", subscriptionFault.getDescription() );

		// case topic not specified
		subscriptionFault = null;
		
			try {
				//         consumerURI = new URI
				// ("file:///usr/cms/borgia/usr/eclipse/workspace/");
				consumerURI = new URI(
						"http://lxgrid04.pd.infn.it:8080/axis/services/CEMonitorConsumerPort" );
			} catch (MalformedURIException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		
		subscription.setMonitorConsumerURL( consumerURI );
		try {
			subscriptionRef = monitorService.subscribe( subscription );
		} catch( SubscriptionFault e ) {
			subscriptionFault = e;
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}
		assertEquals( "Topic not specified", subscriptionFault.getDescription() );

		// case topic not supported
		TopicNotSupportedFault topicFault = new TopicNotSupportedFault( );
		Topic topic = new Topic( );
		topic.setName( "a topic" );
		subscription.setTopic( topic );
		try {
			subscriptionRef = monitorService.subscribe( subscription );
		} catch( SubscriptionFault e ) {
			subscriptionFault = e;
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}
		assertEquals( "Topic \"" + topic.getName( ) + "\" not supported",
				subscriptionFault.getDescription() );

		
		// case no exceptions
		subscriptionRef = null;
		try {

			TopicArray topics = monitorService.getTopics();
			topic = topics.getTopic(0);
//			System.out.println("topic name: "+topic.getName());

		} catch (GenericFault e2) {
			e2.printStackTrace();
		} catch (RemoteException e2) {
			e2.printStackTrace();
		}


		Dialect dialect = new Dialect();
		dialect.setName("Test dialect");
		Dialect[] dialects = new Dialect[1];
		dialects[0]=dialect;
		topic.setDialect(dialects);
		subscription.setTopic( topic );
		Policy policy = new Policy( );
		policy.setRate( 5000 );
		subscription.setPolicy( policy );
		Calendar expirationTime = Calendar.getInstance();
		expirationTime.add(1,1);
		subscription.setExpirationTime(expirationTime);
		
		try {
			subscriptionRef = monitorService.subscribe( subscription );
		} catch( SubscriptionFault e ) {
			e.printStackTrace( );
		} catch( GenericFault e ) {
			e.printStackTrace( );
		} catch( RemoteException e ) {
			e.printStackTrace( );
		}
		assertNotNull(subscriptionRef.getSubscriptionID());

	}

	public void testPauseSubscription() {
		// case subscriptionRef = null
		try {
			monitorService.pauseSubscription(null);
		} catch (org.glite.ce.faults.GenericFault e) {
			genericFault = e;
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertEquals("the subscriptionRef is null",genericFault.getDescription());
		
		
		try {
			monitorService.pauseSubscription(subscriptionRef);
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
	}

	public void testResumeSubscription() {
		// case subscriptionRef = null
		try {
			monitorService.resumeSubscription(null);
		} catch (org.glite.ce.faults.GenericFault e) {
			genericFault = e;
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertEquals("the subscriptionRef is null",genericFault.getDescription());
		
		
		try {
			monitorService.resumeSubscription(subscriptionRef);
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
	}
	
	public void testUnsubscribe() {
		//case subscriptionRef = null
		org.glite.ce.faults.GenericFault fault = null;
		try {
			monitorService.unsubscribe(null);
		} catch (org.glite.ce.faults.GenericFault e) {
			fault = e;
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertEquals("the subscriptionRef is null",fault.getDescription());
		
		
		try {
			monitorService.unsubscribe(subscriptionRef);
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
//		assertNull(subscriptionRef.getSubscriptionID());
		
		try {
			monitorService.unsubscribe(subscriptionRef);
		} catch (org.glite.ce.faults.GenericFault e) {
			genericFault = e;
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertEquals("internal exception",genericFault.getDescription());
	}

	


	public void testGetEvent() {
		// case topic null
		org.glite.ce.faults.GenericFault fault = null;

		try {
			monitorService.getEvent(null);
		} catch (TopicNotSupportedFault e1) {
			e1.printStackTrace();
		} catch (DialectNotSupportedFault e1) {
			e1.printStackTrace();
		} catch (org.glite.ce.faults.GenericFault e1) {
			fault = e1;
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}


		assertEquals("the topicExpression is null",fault.getDescription());
		// case topic not supported
		Topic topic = new Topic();
		topic.setName("test topic");
		
		try {
			monitorService.getEvent(topic);
		} catch (TopicNotSupportedFault e) {
			topicNotSupportedFault = e;
		} catch (DialectNotSupportedFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertEquals("Topic \"test topic\" not supported",topicNotSupportedFault.getDescription());
		
		//TODO other exceptions to be tested
	}

	/*
	 * Class under test for void notify(Notification)
	 */
	public void testNotify_Notification() {
		Notification notification = new Notification();
		notification.setEvent(event);
		try {
			monitorService.notify(notification);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}
	
	

	public void testDoOnResourceEvent() {
		
		// case holder is  a PredefinedSubscriptionHolder instance

		java.net.URI predefinedSubscriptionPath = null;
		try {
			predefinedSubscriptionPath = new java.net.URI("file://"+System.getProperty("sensorURI")+"subscription.xml");
		} catch (URISyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		PredefinedSubscriptionHolder predefinedSubscriptionHolder = new PredefinedSubscriptionHolder(messageContext,predefinedSubscriptionPath);

        SubscriptionPersistent subscription = new SubscriptionPersistent();
        subscription.setName("Test subscription");
        try {
			consumerURI = new URI(
			"http://lxgrid04.pd.infn.it:8080/axis/services/CEMonitorConsumerPort" );
		} catch (MalformedURIException e1) {
			e1.printStackTrace();
		}
		Topic topic = null;
		try {
			TopicArray topics = monitorService.getTopics();
			topic = topics.getTopic(0);
		} catch (GenericFault e2) {
			e2.printStackTrace();
		} catch (RemoteException e2) {
			e2.printStackTrace();
		}
		subscription.setTopic(topic);
        subscription.setMonitorConsumerURL(consumerURI);
        Resource[] resources = new Resource[1];
        resources[0]=subscription;
        Policy policy = new Policy( );
		policy.setRate( 5000 );
		subscription.setPolicy( policy );
		Calendar expirationTime = Calendar.getInstance();
		expirationTime.add(1,1);
		subscription.setExpirationTime(expirationTime);
		//add new resource
		ResourceEvent resourceEvent = new ResourceEvent(predefinedSubscriptionHolder, ResourceEvent.RESOURCE_ADDED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		//update a resource
		resourceEvent = new ResourceEvent(predefinedSubscriptionHolder, ResourceEvent.RESOURCE_UPDATED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		//remove a resource
		resourceEvent = new ResourceEvent(predefinedSubscriptionHolder, ResourceEvent.RESOURCE_REMOVED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		

		// case holder is  a QueryProcessorHolder instance
		
		QueryProcessorHolder queryProcessorHolder = new QueryProcessorHolder(messageContext,predefinedSubscriptionPath,predefinedSubscriptionPath);
		ClassAdProcessor classAdProcessor = new ClassAdProcessor();
		classAdProcessor.setName("Test ClassAdProcessor");
		resources[0]= classAdProcessor;
		//resource added
		resourceEvent = new ResourceEvent(queryProcessorHolder, ResourceEvent.RESOURCE_ADDED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		//resource updated
		resourceEvent = new ResourceEvent(queryProcessorHolder, ResourceEvent.RESOURCE_UPDATED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		//resource removed
		resourceEvent = new ResourceEvent(queryProcessorHolder, ResourceEvent.RESOURCE_REMOVED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		
		// case holder is an ActionHolder instance
		ActionHolder actionHolder = new ActionHolder(messageContext,predefinedSubscriptionPath,predefinedSubscriptionPath);
		Action action = new Action();
		action.setName("Test action");
		resources[0]=action;
		//resource added
		resourceEvent = new ResourceEvent(actionHolder, ResourceEvent.RESOURCE_ADDED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		//resource updated
		resourceEvent = new ResourceEvent(actionHolder, ResourceEvent.RESOURCE_UPDATED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		//resource removed
		resourceEvent = new ResourceEvent(actionHolder, ResourceEvent.RESOURCE_REMOVED, resources);
		monitorService.doOnResourceEvent(resourceEvent);

		// case holder is an SensorHolder instance
		SensorHolder sensorHolder = new SensorHolder(messageContext,predefinedSubscriptionPath,predefinedSubscriptionPath);
		TestSensor testSensor = new TestSensor();
		resources[0]=testSensor;
		//resource added
		// already done in testDoOnSensorEvent()
//		resourceEvent = new ResourceEvent(sensorHolder, ResourceEvent.RESOURCE_ADDED, resources);
//		monitorService.doOnResourceEvent(resourceEvent);
		//resource updated
		resourceEvent = new ResourceEvent(sensorHolder, ResourceEvent.RESOURCE_UPDATED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		//resource removed
		resourceEvent = new ResourceEvent(sensorHolder, ResourceEvent.RESOURCE_REMOVED, resources);
		monitorService.doOnResourceEvent(resourceEvent);
		
		}
//	
	
}


