/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */


import org.glite.ce.monitor.action.DoNotSendNotificationAction;
import org.glite.ce.monitor.action.SendNotificationAction;
import org.glite.ce.monitorapij.queryprocessor.QueryResult;
import org.glite.ce.monitorapij.sensor.SensorException;
import org.glite.ce.monitorapij.types.Dialect;
import org.glite.ce.monitorapij.types.Notification;
import org.glite.ce.monitorapij.types.Topic;

import junit.framework.TestCase;

public class DoNotSendNotificationActionTest extends TestCase {

	DoNotSendNotificationAction doNotsendNotificationAction = null;
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		doNotsendNotificationAction = new DoNotSendNotificationAction();
	}

	public void testSendNotificationAction() {
		assertNotNull(doNotsendNotificationAction);
	}

	/*
	 * Class under test for void execute(Notification, QueryResult[])
	 */
	public void testExecute_NotificationQueryResultArray() {
		String exception = "";
		QueryResult[] queryResult = new QueryResult[2];
		queryResult[0] = new QueryResult(true,"test","this is a test");
		queryResult[1] = new QueryResult(false,"test","this is a test");
		Notification notification = new Notification();
		TestSensor testSensor = new TestSensor();
        testSensor.addFormat(new TestFormat("test format"));
	    testSensor.setName("Test Sensor");
		TestSensorEvent testSensorEvent = new TestSensorEvent( testSensor, System.currentTimeMillis( ), new String[]{"Test Event"} );
		testSensorEvent.addParameter("first","first test");
		try {
            testSensorEvent.applyFormat("test format");
        } catch (SensorException e1) {
            e1.printStackTrace();
        }
        

		Topic topic = new Topic();
		Dialect[] dialects = new Dialect[2];
	    topic.setName("TEST");
	    Dialect dialect = new Dialect();
		dialect.setName(null);
		dialects[0]=dialect;
		dialect.setName(null);
		dialects[1]=dialect;
		// dialect null

		topic.setDialect(dialects);
//		
		notification.setTopic(topic);
		notification.setEvent(testSensorEvent);
		try {
			doNotsendNotificationAction.execute(notification,queryResult);
		} catch (Exception e) {

			exception = e.getMessage();
		}
		assertEquals("invalid dialect!", exception);
		

		dialect.setName("test dialect");
		dialects[0]=dialect;
		dialects[1]=dialect;
		topic.setDialect(dialects);
		notification.setTopic(topic);
		
		try {
			doNotsendNotificationAction.execute(notification,queryResult);
		} catch (Exception e) {
			exception = e.getMessage();
		}
		assertEquals("invalid dialect! (\"test dialect\")", exception);
		

		// no exceptions
		dialect.setName("ISM_LDIF");
		dialects[0]=dialect;
		dialects[1]=dialect;
		topic.setDialect(dialects);
		notification.setTopic(topic);
		
		try {
			doNotsendNotificationAction.execute(notification,queryResult);
		} catch (Exception e) {
			e.printStackTrace();
		}

		
	}
}
