/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPException;

import org.apache.axis.AxisFault;
import org.apache.axis.Handler;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.SOAPPart;
import org.apache.axis.handlers.soap.SOAPService;
import org.apache.axis.message.EnvelopeHandler;
import org.apache.axis.message.SOAPBody;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.server.AxisServer;
import org.glite.ce.monitor.Logger;

import junit.framework.TestCase;

public class LoggerTest extends TestCase {
   
   private static MessageContext messageContext;
   private static AxisServer axisServer;
   private static Logger logger ;
   

   public static void main( String[] args ) {
      junit.textui.TestRunner.run( LoggerTest.class );
   }
   
   public void setUp(){
   	logger = new Logger();
    setUpMessageContext("file:/../org.glite.ce.ceplugin/",
          "1.0","Testing CE Monitor Service",System.getProperty("test.conf.dir")+"/subscription.reg",
          "subscription.xml");
   }
   
   
   protected void setUpMessageContext(String sensorURI, String monitorVersion, String monitorDescription, String subscriptionRegistryURI,String predifinedSubscriptionListPath){
      messageContext = new MessageContext(axisServer);
      messageContext.setProperty( "sensorURI", sensorURI);
      messageContext.setProperty( "monitorVersion", monitorVersion );
      messageContext.setProperty( "monitorDescription", monitorDescription );
      messageContext.setProperty( "subscriptionRegistryURI", subscriptionRegistryURI );
      messageContext.setProperty( "predefinedSubscriptionListPath", predifinedSubscriptionListPath );
      
   }
   
   /*
    * Class under test for void invoke(MessageContext)
    */
   public void testInvokeMessageContext( ) {
      

   	AxisFault axisFault = null;
   	//TODO both functions need to avoid error while logger tries msgContext.getCurrentMessage( ).getSOAPPartAsString( );
     // this unit test is not complete!! 
      
      // case option filename null
      try {
         logger.invoke(messageContext);
      } catch( AxisFault e ) {
         axisFault = e;
      }
      assertEquals("No log file configured for the LogHandler!",axisFault.getFaultReason() );
      
      // case option filename = ""
      axisFault=null;
      logger.setOption("filename","");
      try {
         logger.invoke(messageContext);
      } catch( AxisFault e ) {
         axisFault = e;
      }
      assertEquals("No log file configured for the LogHandler!",axisFault.getFaultReason() );
      

      logger.setOption("filename",System.getProperty("test.log.dir")+"/logfile");
      Integer numAccesses = new Integer(0);
      
      SOAPService soapService = new SOAPService();
      try {
         messageContext.setService(soapService);
      } catch( AxisFault e1 ) {
         e1.printStackTrace();
      }
      soapService.setOption("accesses",numAccesses);
      SOAPEnvelope soapEnvelope = new SOAPEnvelope();
      Message message = new Message(soapEnvelope,true);
      message.setContentDescription("test Message");
      messageContext.setCurrentMessage(message);
      try {
         logger.invoke(messageContext);
      } catch( AxisFault e ) {
         axisFault = e;
//         e.printStackTrace();
      }
   }
   
   
   
   
   public void testOnFault(){
   	SOAPEnvelope soapEnvelope = new SOAPEnvelope();
//	Message message = new Message(soapEnvelope,mimeHeaders);
   	Message message = new Message("Test");
    messageContext.setCurrentMessage(message);
   	
    logger.setOption("filename",System.getProperty("test.log.dir")+"/logfile");
   	SOAPService soapService = new SOAPService();
   	soapService.setOption("accesses",new Integer(1));
   	
   	try {
		messageContext.setService(soapService);
		System.out.println(message.getSOAPPartAsString());
//		System.out.println(message.getSOAPEnvelope());
		logger.onFault(messageContext);
	} catch (AxisFault e) {
		e.printStackTrace();
	}
	

   	
   	
   }

}
