/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import java.net.URI;

import javax.xml.namespace.QName;

import org.apache.axis.MessageContext;
import org.apache.axis.encoding.DeserializerFactory;
import org.apache.axis.encoding.TypeMapping;
import org.apache.axis.encoding.TypeMappingImpl;
import org.apache.axis.encoding.TypeMappingRegistryImpl;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.server.AxisServer;
import org.glite.ce.monitor.holder.QueryProcessorHolder;
import org.glite.ce.monitor.queryprocessor.ClassAdProcessor;
import org.glite.ce.monitorapij.queryprocessor.QueryProcessor;
import org.glite.ce.monitorapij.types.CEMonResource;
import org.glite.ce.monitorapij.types.Action;

import junit.framework.TestCase;

public class QueryProcessorHolderTest extends TestCase {

	private MessageContext messageContext =null;
	private AxisServer axisServer = null;
	private QueryProcessorHolder queryProcessorHolder = null;
	private String path;
	private ClassAdProcessor queryProcessor = null;
	private QueryProcessor[] queryProcessors = null;
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		setUpMessageContext();
	  	path = "file://"+(String)messageContext.getProperty( "resourceURI" );
	  	URI configPath = new URI( path );
	  	path = "file://"+(String)messageContext.getProperty( "sensorConfigURI" );
	  	URI queryProcessorPath = new URI( path );
		queryProcessorHolder = new QueryProcessorHolder(messageContext,queryProcessorPath,configPath);
	}

	public void testQueryProcessorHolder() {

		assertNotNull(queryProcessorHolder);
	}

	public void testGetQueryProcessor() {
		ClassAdProcessor result = (ClassAdProcessor) queryProcessorHolder.getQueryProcessor(0);
		assertNull(result);
		
		queryProcessor = new ClassAdProcessor();
		queryProcessor.setId(1);
		queryProcessorHolder.addResource(queryProcessor);
	   	result = (ClassAdProcessor) queryProcessorHolder.getQueryProcessor(1);
	   	assertEquals(1,result.getId().intValue());
	   	queryProcessorHolder.removeResource(queryProcessor);
	}

	/*
	 * Class under test for QueryProcessor[] getQueryProcessors()
	 */
	public void testGetQueryProcessors() {
	    QueryProcessor[] result = null;
	    result = queryProcessorHolder.getQueryProcessors( );
	   	assertEquals(0,result.length);

	   	ClassAdProcessor queryProcessor = new ClassAdProcessor();
	   	queryProcessor.setId(1);
	   	queryProcessorHolder.addResource(queryProcessor);
	   	queryProcessors = queryProcessorHolder.getQueryProcessors( );
	   	assertEquals(1,queryProcessors.length);
	   	queryProcessorHolder.removeResource(queryProcessor);
	}

	/*
	 * Class under test for QueryProcessor[] getQueryProcessors(String)
	 */
	public void testGetQueryProcessorsString() {
		queryProcessor = new ClassAdProcessor();
		queryProcessor.setId(1);
		queryProcessor.setName("test");
		queryProcessorHolder.addResource(queryProcessor);
	   	queryProcessors = queryProcessorHolder.getQueryProcessors("test" );
	   	assertEquals(1,queryProcessors.length);
	   	assertEquals("test",queryProcessors[0].getName());
	   	queryProcessorHolder.removeResource(queryProcessor);
	}

	 protected void setUpMessageContext( ) {
	 	//TODO rivedere tutti i path
		messageContext = new MessageContext( axisServer );
		messageContext.setProperty( "subscriptionRegistryURI",System.getProperty("sensorURI")+"subscription.reg" );
		messageContext.setProperty( "resourceURI", "file://"+System.getProperty("sensorURI" ));
		messageContext.setProperty( "sensorConfigURI", "file://"+System.getProperty("sensorURI" )+"sensorConfig.xml" );
		messageContext.setProperty( "queryProcessorURI","file://"+ System.getProperty("sensorURI" ) );
		messageContext.setProperty( "queryProcessorConfigURI", "file://"+System.getProperty("sensorURI" )+"queryProcessorConfig.xml");
		messageContext.setProperty( "actionURI","file://"+ System.getProperty("sensorURI" ) );
		messageContext.setProperty( "actionConfigURI", "file://"+System.getProperty("sensorURI" )+"actionConfig.xml" );

		messageContext.setProperty( "monitorVersion","1.0" );
		messageContext.setProperty( "monitorDescription","First CE-Monitor prototype!" );
		messageContext.setProperty( "sensorURI","file://"+System.getProperty("sensorURI" ) );
		TypeMappingRegistryImpl tmr = new TypeMappingRegistryImpl();
		//TypeMapping typeMapping = null;
		TypeMappingImpl tmi = new TypeMappingImpl();


		QName testSensorQName = new QName("TestSensor");
		QName qName = new QName("CEMonResource");
		BeanSerializerFactory bsf = new BeanSerializerFactory(CEMonResource.class, qName);
		BeanDeserializerFactory bdsf = new BeanDeserializerFactory(CEMonResource.class, qName);
//		SerializerFactory sf = (SerializerFactory) rh;
//		DeserializerFactory dsf = (DeserializerFactory) rh;
		tmi.register(CEMonResource.class,qName,bsf,bdsf);
//		QName qName2 = tmi.getTypeQName(CEMonResource.class);
//		System.out.println("QNAME"+qName2.toString());
		DeserializerFactory bdf;
//		tmi.register(TestSensor.class,testSensorQName,bs,bdf);
		//tmr.register("test",tmi);
		messageContext.setTypeMappingRegistry(tmr);
	 }

}
