/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.Hashtable;

import org.glite.ce.monitor.registry.ResourceRegistry;
import org.glite.ce.monitorapij.types.Subscription;

import junit.framework.TestCase;

public class ResourceRegistryTest extends TestCase {
   
   private static ResourceRegistry resourceRegistry;
   private String filename = null;
   private Subscription subscription = null;
   private Subscription result = null;
   private Hashtable registry = new Hashtable( );

   public static void main( String[] args ) {
      junit.textui.TestRunner.run( ResourceRegistryTest.class );
   }
   
   private void cleanTestReg(String file) throws IOException, ClassNotFoundException{
      
      try {
         FileInputStream fis = new FileInputStream( file );
         ObjectInputStream ois = new ObjectInputStream( fis );
         registry = (Hashtable)ois.readObject( );
         ois.close( );
         fis.close( );
         registry.clear();
      } catch( FileNotFoundException ex ) {
         registry.clear();
         FileOutputStream fos = new FileOutputStream( file );
         ObjectOutputStream oos = new ObjectOutputStream( fos );
         oos.writeObject( registry );
         oos.close( );
         fos.close( );
      }
      registry.clear();
      FileOutputStream fos = new FileOutputStream( file );
      ObjectOutputStream oos = new ObjectOutputStream( fos );
      oos.writeObject( registry );
      oos.close( );
      fos.close( );
      
   }


   /*
    * @see TestCase#setUp()
    */
   protected void setUp( ) throws Exception {
            
      //    TODO change to the effective path
      
      filename = System.getProperty("local_sensor_uri")+"/test.reg";
      cleanTestReg(filename);
      try {
         resourceRegistry = new ResourceRegistry(filename);
      } catch( FileNotFoundException e ) {
         e.printStackTrace();
      } catch( Exception e ) {
         e.printStackTrace();
      }
      
   }


   /*
    * Class under test for void ResourceRegistry()
    */
   public void testResourceRegistry( ) {
      // default.reg
      try {
         resourceRegistry = new ResourceRegistry();
      } catch( FileNotFoundException e ) {
         e.printStackTrace();
      } catch( Exception e ) {
         e.printStackTrace();
      }
      assertNotNull(resourceRegistry);
      
      
   }


   /*
    * Class under test for void ResourceRegistry(String)
    */
   public void testResourceRegistryString( ) {
      //    filename = null
      filename = null;
      try {
         resourceRegistry = new ResourceRegistry(filename);
      } catch( FileNotFoundException e ) {
         e.printStackTrace();
      } catch( Exception e ) {
         e.printStackTrace();
      }
      assertNotNull(resourceRegistry);
      
     
   }


   public void testLoad( ) {
      try {
         resourceRegistry.load();
      } catch( FileNotFoundException e ) {
         e.printStackTrace();
      } catch( Exception e ) {
         e.printStackTrace();
      }
   }


   public void testSave( ) {
      try {
         resourceRegistry.save();
      } catch( FileNotFoundException e ) {
         e.printStackTrace();
      } catch( Exception e ) {
         e.printStackTrace();
      }
   }


   /*
    * Class under test for void register(String, Object)
    */
   public void testRegisterStringObject( ) {
      subscription = new Subscription();
      try {
         resourceRegistry.register("Test Subscription",subscription) ;
      } catch( Exception e ) {
         e.printStackTrace();
      }
      try {
         resourceRegistry.unregister("Test Subscription");
      } catch( Exception e1 ) {
         e1.printStackTrace();
      }
   }


   /*
    * Class under test for void register(String, Object, boolean)
    */
   public void testRegisterStringObjectboolean( ) {
      subscription = new Subscription();
      Exception exception = null;
      try {
         resourceRegistry.register("Test Subscription",subscription,true) ;
      } catch( Exception e ) {
         e.printStackTrace();
      }
      // trying to register again the same object
      try {
         resourceRegistry.register("Test Subscription",subscription,true) ;
      } catch( Exception e ) {
         exception = e;
      }
      assertEquals("java.lang.Exception: The obj to be registered already exist!",exception.toString());
      
      // remove subscription
      try {
         resourceRegistry.unregister("Test Subscription");
      } catch( Exception e1 ) {
         e1.printStackTrace();
      }
      // another subscription not persistent
      try {
         resourceRegistry.register("Test Subscription",subscription,false) ;
      } catch( Exception e ) {
         exception = e;
      }
   }


   public void testUnregister( ) {
      // trying to unregister an inesistent entry
      Exception exception = null;
      try {
         resourceRegistry.unregister("Test Subscription");
      } catch( Exception e ) {
         exception = e;
      }
      assertEquals("java.lang.Exception: The obj to be unregistered do not exists!",exception.toString());
   }


   public void testGetResource( ) {
      // key = null
      String key = null;
      Subscription result = new Subscription();
      assertNotNull(result);
      result = (Subscription) resourceRegistry.getResource(key);
      assertNull(result);
      //    register an object than retreive it
      result = null;
      Subscription subscription = new Subscription();
      try {
         resourceRegistry.register("Test sub",subscription);
      } catch( Exception e ) {
         e.printStackTrace();
      }

      result = (Subscription) resourceRegistry.getResource("Test sub");
      assertEquals(subscription.toString(),result.toString());
   }


   public void testGetAllResources( ) {
      // registry a new subscription than retreive it
      
      Collection collection = null;
      Subscription subscription = new Subscription();
      try {
         resourceRegistry.register("Test",subscription);
      } catch( Exception e ) {
         e.printStackTrace();
      }
      collection = resourceRegistry.getAllResources();
      assertTrue(collection.contains(subscription));
      //    remove subscription 
      try {
         resourceRegistry.unregister("Test");
      } catch( Exception e ) {
         e.printStackTrace();
      }
      collection = resourceRegistry.getAllResources();
      assertFalse(collection.contains(subscription));
   }



}
