/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */
import java.net.URI;

import org.apache.axis.MessageContext;
import org.apache.axis.server.AxisServer;
import org.apache.axis.types.URI.MalformedURIException;
import org.glite.ce.monitor.holder.SensorHolder;
import org.glite.ce.monitorapij.resource.Resource;
import org.glite.ce.monitorapij.resource.ResourceListener;
import org.glite.ce.monitorapij.sensor.Sensor;
import org.glite.ce.monitorapij.sensor.SensorListener;

import junit.framework.TestCase;


public class SensorHolderTest extends TestCase {
   
   private SensorHolder sensorHolder = null;
   private TestSensor testSensor= null;
   private SensorListener listener = null;
   private Exception exception = null;
   private Sensor[] sensors = null;
   private Sensor sensor = null;
   private MessageContext messageContext = null;
   private AxisServer axisServer = null;
   private String path ="";
   private URI resourcePath = null;
   private URI resourceConfigPath = null;

   /*
    * @see TestCase#setUp()
    */
   protected void setUp( ) throws Exception {
   	TestUtilities testUtilities = new TestUtilities();
   	messageContext= testUtilities.getMessageContext();
   	path = (String)messageContext.getProperty( "sensorURI" );
   	resourcePath = new URI( path );
   	path = (String)messageContext.getProperty( "sensorConfigURI" );
   	resourceConfigPath = new URI( path );

   	// i need a good sensorConfig.xml file
//    sensorHolder = new SensorHolder( messageContext, resourcePath, resourceConfigPath);
  sensorHolder = new SensorHolder( messageContext, resourcePath, null);
//      sensorHolder.removeAllSensors();
   }


   public void testSensorHolder( ) {
      assertNotNull(sensorHolder);
   }

   

   public void testGetSensor(){
   	sensor = sensorHolder.getSensor(1);
   	assertNull(sensor);
   	testSensor = new TestSensor();
   	testSensor.setId(1);
   	sensorHolder.addResource(testSensor);
   	sensor = sensorHolder.getSensor(1);
   	assertNotNull(sensor);
   }

   public void testGetSensorByType( ) {
   	TestSensor testSensor = new TestSensor();
   	testSensor.setId(1);
   	testSensor.setType("test sensor");
   	sensorHolder.addResource(testSensor);
   	// asking for a not esistent type
   	sensor = (TestSensor) sensorHolder.getSensorByType("nothing");
   	assertEquals(null,sensor);
   	sensor = sensorHolder.getSensorByType("test sensor");
   	assertEquals ("test sensor",sensor.getType());
    }


   /*
    * Class under test for Sensor[] getSensors()
    */
   public void testGetSensors( ) {
   
   	Sensor[] sensors = sensorHolder.getSensors( );
   	assertEquals(0,sensors.length);
   	testSensor = new TestSensor();
   	testSensor.setId(1);
   	sensorHolder.addResource(testSensor);
   	sensors = sensorHolder.getSensors( );
   	assertEquals(1,sensors.length);
   	sensorHolder.removeResource(testSensor);
   }



	public void testRun() {

		sensorHolder.run();
	}

	/*
	 * Class under test for void ResourceHolder(MessageContext, URI)
	 */
	public void testResourceHolderMessageContextURI() {
	}

	/*
	 * Class under test for void ResourceHolder(MessageContext, URI, URI)
	 */
	public void testResourceHolderMessageContextURIURI() {
	}

	public void testGetResourceURI() {
		URI resURI = sensorHolder.getResourceURI();
		assertEquals(resURI,resourcePath);
	}

	public void testGetConfigFileURI() {
		URI configURI = sensorHolder.getConfigFileURI();
		assertEquals(null,configURI);
	}


	public void testGetResourceByType() {
		TestSensor testSensor = new TestSensor();
		testSensor.setId(1);
		testSensor.setType("test sensor");
		sensorHolder.addResource(testSensor);
		// asking for a not esistent type
		Resource[] resources  = (Resource[]) sensorHolder.getResourceByType("nothing");
		assertEquals(0,resources.length);
		resources = sensorHolder.getResourceByType("test sensor");
		assertEquals ("test sensor",resources[0].getType());
    
	}

	public void testGetResourceByJARPath() {
		TestSensor testSensor = new TestSensor();
		testSensor.setId(1);
		org.apache.axis.types.URI testURI= null;
		try {
			testURI = new org.apache.axis.types.URI(path = (String)messageContext.getProperty( "sensorURI" ));
		} catch (MalformedURIException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		testSensor.setJarPath(testURI);
		sensorHolder.addResource(testSensor);
		// asking for a not esistent type
		Resource[] resources  = (Resource[]) sensorHolder.getResourceByJARPath(new org.apache.axis.types.URI());
		assertEquals(0,resources.length);
		resources = sensorHolder.getResourceByJARPath(testURI);
		assertEquals (testURI,resources[0].getJarPath());
	}

	public void testGetResourceByName() {
		TestSensor testSensor = new TestSensor();
		testSensor.setId(1);
		String name = "test";
		testSensor.setName(name);
		sensorHolder.addResource(testSensor);
		// asking for a not esistent type
		Resource[] resources  = (Resource[]) sensorHolder.getResourceByName("nothing");
		assertEquals(0,resources.length);
		resources = sensorHolder.getResourceByName(name);
		assertEquals (name,resources[0].getName());
	}

	public void testSetResources() {
		Resource[] resourceList = sensorHolder.getResources();
		assertEquals(0,resourceList.length);
		TestSensor testSensor = new TestSensor();
		testSensor.setId(1);
		String name = "test";
		testSensor.setName(name);
		Resource[] resources = new Resource[1];
		resources[0] = (Resource) testSensor;
		sensorHolder.setResources(resources);
		resourceList = sensorHolder.getResources();
		assertEquals(1,resourceList.length);
		sensorHolder.clear();
		resourceList = sensorHolder.getResources();
		assertEquals(0,resourceList.length);
	}



	public void testGetSensorListeners() {
		ResourceListener[] sensorListeners = sensorHolder.getSensorListeners();
		assertEquals(0,sensorListeners.length);
	}


	public void testRemoveResourceListener() {
	}

	public void testFireResourceEvent() {
	}

}

