/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import java.util.Calendar;
import java.util.regex.Pattern;

import org.apache.axis.types.URI;
import org.apache.axis.types.URI.MalformedURIException;
import org.glite.ce.monitor.SubscriptionPersistent;
import org.glite.ce.monitorapij.types.Policy;
import org.glite.ce.monitorapij.types.Query;
import org.glite.ce.monitorapij.types.Subscription;
import org.glite.ce.monitorapij.types.Topic;

import junit.framework.TestCase;

public class SubscriptionPersistentTest extends TestCase {
   
   private static Subscription subscription;
   private static SubscriptionPersistent subscriptionPersistent;
   private static URI consumerURI;
   private static Topic topic;

   public static void main( String[] args ) {
      junit.textui.TestRunner.run( SubscriptionPersistentTest.class );
   }


   public void testSubscriptionPersistent( ) { 
   	
   	try {
		subscriptionPersistent = new SubscriptionPersistent();
	} catch (Exception e3) {
		e3.printStackTrace();
	}
	assertNotNull(subscriptionPersistent);
	
      // case subscription null
      Exception exception = null;
      subscription = null;
      try {
         subscriptionPersistent = new SubscriptionPersistent(subscription);
      } catch( Exception e ) {
         exception=e;
      }
      assertEquals("java.lang.Exception: the subscription is null", exception.toString());
      
      
      // no exception
      subscription = new Subscription();
      try {
         consumerURI = new URI ("http://lxgrid04.pd.infn.it:8080/axis/services/CEMonitorConsumerPort");
      } catch( MalformedURIException e1 ) {
         e1.printStackTrace();
      }
      subscription.setMonitorConsumerURL( consumerURI );
      Topic topic = new Topic();
      topic.setName("TEST");
      subscription.setTopic(topic);
      Policy policy = new Policy();
      policy.setRate(5000);
      subscription.setPolicy(policy);
      try {
         subscriptionPersistent = new SubscriptionPersistent(subscription);
      } catch( Exception e2 ) {
         e2.printStackTrace();
      }
      // verify the result by looking at subscription ID
      assertNotNull(subscriptionPersistent.getID());
   }


   public void testGetRetryCount(){
   	// default value = 20
   	assertEquals(20,subscriptionPersistent.getRetryCount());
   }
   
   public void testSetRetryCount(){
   	subscriptionPersistent.setRetryCount(30);
   	// default value = 20
   	assertEquals(30,subscriptionPersistent.getRetryCount());
   }

   public void testDecrRetryCount( ) {
   	subscriptionPersistent.setRetryCount(30);
    subscriptionPersistent.decrRetryCount();
    assertEquals(29,subscriptionPersistent.getRetryCount());
 }
   public void testGetID( ) {
      assertNotNull(subscriptionPersistent.getID());
   }


   public void testGetPauseTime( ) {
   }


   public void testResume( ) {
      subscriptionPersistent.pause();
      assertTrue(subscriptionPersistent.isPaused());
      subscriptionPersistent.resume();
      assertFalse(subscriptionPersistent.isPaused());
   }


   /*
    * Class under test for void pause()
    */
   public void testPause( ) {
      subscriptionPersistent.resume();
      assertFalse(subscriptionPersistent.isPaused());
      subscriptionPersistent.pause();
      assertTrue(subscriptionPersistent.isPaused());
   }


   /*
    * Class under test for void pause(Calendar)
    */
   public void testPauseCalendar( ) {
      Calendar currentTime = Calendar.getInstance( );
      subscriptionPersistent.pause(currentTime);
      assertEquals(currentTime,subscriptionPersistent.getPauseTime());

   }


   public void testIsPaused( ) {
      // case pauseTime = null
      subscriptionPersistent.resume();
      assertFalse(subscriptionPersistent.isPaused());
      Calendar pauseTime = null;
      subscriptionPersistent.pause(pauseTime);
      assertTrue(subscriptionPersistent.isPaused());
      

      subscriptionPersistent.resume();
      assertFalse(subscriptionPersistent.isPaused());
      Calendar currentTime = Calendar.getInstance( );
      currentTime.add(Calendar.DATE,1);
      subscriptionPersistent.pause(currentTime);
      assertTrue(subscriptionPersistent.isPaused());
      
      subscriptionPersistent.pause();
      assertTrue(subscriptionPersistent.isPaused());
      currentTime.add(Calendar.DATE,-2);
      subscriptionPersistent.pause(currentTime);
      assertFalse(subscriptionPersistent.isPaused());
   }


   public void testGetCreationTime( ) {
      Calendar creationTime = null;
      creationTime = subscriptionPersistent.getCreationTime();
      assertNotNull(creationTime);
   }


   public void testIsExpired( ) {
      // case expiration time = null
      assertFalse(subscriptionPersistent.isExpired());
      
      // this will expire in one day
      
      Calendar currentTime = Calendar.getInstance( );
      currentTime.add(Calendar.DATE,1);
      subscription.setExpirationTime(currentTime);
      try {
         subscriptionPersistent = new SubscriptionPersistent(subscription);
      } catch( Exception e ) {
         e.printStackTrace();
      }
      assertFalse(subscriptionPersistent.isExpired());
      
      //    this expired yesterday
      currentTime.add(Calendar.DATE,-2);
      subscription.setExpirationTime(currentTime);
      try {
         subscriptionPersistent = new SubscriptionPersistent(subscription);
      } catch( Exception e ) {
         e.printStackTrace();
      }
      assertTrue(subscriptionPersistent.isExpired());
   }

//
//   public void testGetRegExPattern( ) {
//      Pattern pattern = null;
//      Policy policy = new Policy();
////      policy.setRegex("all");
//      subscriptionPersistent.setPolicy(policy);
//      pattern = subscriptionPersistent.getRegExPattern();
//      assertNotNull(pattern);
//   }


//   /*
//    * Class under test for void setPolicy(Policy)
//    */
//   public void testSetPolicyPolicy( ) {
//      // case policy = null
//      Policy policy = null;
//      subscriptionPersistent.setPolicy(policy);
//      
//      policy = new Policy();
//      Query query = new Query();
//      query.setQueryLanguage("REGEX");
//      query.setExpression("test expression");
//      policy.setQuery(query);
//      policy.setRate(10000);
//      subscriptionPersistent.setPolicy(policy);
//      assertEquals(10000,subscriptionPersistent.getPolicy().getRate());
//      
//      
//   }

}
