/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Calendar;
import java.util.Hashtable;

import org.apache.axis.types.URI;
import org.apache.axis.types.URI.MalformedURIException;
import org.glite.ce.monitor.SubscriptionPersistent;
import org.glite.ce.monitor.registry.SubscriptionRegistry;
import org.glite.ce.monitorapij.types.Policy;
import org.glite.ce.monitorapij.types.Subscription;
import org.glite.ce.monitorapij.types.Topic;

import junit.framework.TestCase;


public class SubscriptionRegistryTest extends TestCase {
   
   private SubscriptionRegistry subscriptionRegistry = null;
   private Hashtable registry= null;
   private Subscription subscription = null;
   private Exception exception = null;
   private SubscriptionPersistent subscriptionPersistent = null;
   private Calendar currentTime;
   private Topic topic = null;
   
   
   public static void main( String[] args ) {
      junit.textui.TestRunner.run( SubscriptionRegistryTest.class );
   }
   
   private void cleanTestReg(String file) throws IOException, ClassNotFoundException{
      
      try {
         FileInputStream fis = new FileInputStream( file );
         ObjectInputStream ois = new ObjectInputStream( fis );
         registry = (Hashtable)ois.readObject( );
         ois.close( );
         fis.close( );
         registry.clear();
      } catch( FileNotFoundException ex ) {
         registry.clear();
         FileOutputStream fos = new FileOutputStream( file );
         ObjectOutputStream oos = new ObjectOutputStream( fos );
         oos.writeObject( registry );
         oos.close( );
         fos.close( );
      }
      registry.clear();
      FileOutputStream fos = new FileOutputStream( file );
      ObjectOutputStream oos = new ObjectOutputStream( fos );
      oos.writeObject( registry );
      oos.close( );
      fos.close( );
      
   }

   
   private Subscription SetUpSubscription(){
      subscription = new Subscription();
      URI consumerURI = null;
      try {
         consumerURI = new URI ("http://lxgrid04.pd.infn.it:8080/axis/services/CEMonitorConsumerPort");
      } catch( MalformedURIException e1 ) {
         e1.printStackTrace();
      }
      subscription.setMonitorConsumerURL( consumerURI );
      Topic topic = new Topic();
      topic.setName("TEST");
      subscription.setTopic(topic);
      Policy policy = new Policy();
      policy.setRate(5000);
      subscription.setPolicy(policy);
      return subscription;
   }
   
   
   /*
    * @see TestCase#setUp()
    */
   protected void setUp( ) throws Exception {
       String testRegisterName=System.getProperty("local_sensor_uri")+"/test.reg";
      cleanTestReg(testRegisterName);
      subscriptionRegistry = new SubscriptionRegistry(testRegisterName);
   }


   /*
    * Class under test for void SubscriptionRegistry()
    */
   public void testSubscriptionRegistry( ) {
      try {
         subscriptionRegistry = new SubscriptionRegistry();
      } catch( FileNotFoundException e ) {
         e.printStackTrace();
      } catch( Exception e ) {
         e.printStackTrace();
      }
      assertNotNull(subscriptionRegistry);
   }


   /*
    * Class under test for void SubscriptionRegistry(String)
    */
   public void testSubscriptionRegistryString( ) {
      try {
         subscriptionRegistry = new SubscriptionRegistry(System.getProperty("local_sensor_uri")+"/test.reg");
      } catch( FileNotFoundException e ) {
         e.printStackTrace();
      } catch( Exception e ) {
         e.printStackTrace();
      }
      assertNotNull(subscriptionRegistry);
   }


   /*
    * Class under test for SubscriptionPersistent register(Subscription)
    */
   public void testRegister_Subscription( ) {
      subscription = SetUpSubscription();
      currentTime = Calendar.getInstance( );
      currentTime.add(Calendar.DATE,2);
      subscription.setExpirationTime(currentTime);
      try {
         subscriptionPersistent = subscriptionRegistry.register(subscription);
      } catch( Exception e1 ) {
         e1.printStackTrace();
      }
      assertNotNull(subscriptionPersistent.getID());
   }


   /*
    * Class under test for SubscriptionPersistent register(Subscription, boolean)
    */
   public void testRegister_Subscriptionboolean( ) {
      // subscription = null
      subscription = null;
      exception = null;
      try {
         subscriptionRegistry.register(subscription,true);
      } catch( Exception e ) {
         exception = e;
      }
      assertEquals("java.lang.Exception: the subscription is null",exception.toString());
      
      // subscription expired
      subscription = SetUpSubscription();
      currentTime = Calendar.getInstance( );
      currentTime.add(Calendar.DATE,-1);
      subscription.setExpirationTime(currentTime);
      
      try {
         subscriptionPersistent = subscriptionRegistry.register(subscription,true);
      } catch( Exception e ) {
         exception = e;
      }
      assertEquals("java.lang.Exception: the subscription is expired!",exception.toString());
      
      // no exceptions
      currentTime.add(Calendar.DATE,2);
      subscription.setExpirationTime(currentTime);
      try {
         subscriptionPersistent = subscriptionRegistry.register(subscription,true);
      } catch( Exception e1 ) {
         e1.printStackTrace();
      }
      assertNotNull(subscriptionPersistent.getID());
   }


   public void testGetSubscription( ) {
      // before i register a subscription
      subscription = SetUpSubscription();
      currentTime = Calendar.getInstance( );
      currentTime.add(Calendar.DATE,2);
      subscription.setExpirationTime(currentTime);
      try {
         subscriptionPersistent = subscriptionRegistry.register(subscription,true);
      } catch( Exception e1 ) {
         e1.printStackTrace();
      }
      String key = subscriptionPersistent.getID( );
      // then get the registered subscription
      subscriptionPersistent = subscriptionRegistry.getSubscription(key);
      assertEquals(key,subscriptionPersistent.getID());
   }


   public void testGetSubscriptionByTopic( ) {
      SubscriptionPersistent[] subscriptionPersistentArray = null; 
      // case topic = null
      topic = null;
      subscriptionPersistentArray = subscriptionRegistry.getSubscriptionByTopic(topic);
      assertNull(subscriptionPersistentArray);
      
      
      //    before i register a subscription
      subscription = SetUpSubscription();
      currentTime = Calendar.getInstance( );
      currentTime.add(Calendar.DATE,2);
      subscription.setExpirationTime(currentTime);
      try {
         subscriptionPersistent = subscriptionRegistry.register(subscription,true);
      } catch( Exception e1 ) {
         e1.printStackTrace();
      }
      //    then get the registered subscription
      subscriptionPersistentArray = subscriptionRegistry.getSubscriptionByTopic(subscription.getTopic());
      assertEquals(subscriptionPersistent.getID(),subscriptionPersistentArray[0].getID());
      
   }
   
   

}
